/**************************************************************************
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.09.2017  mifi  First Version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <windows.h>
#include <stdio.h>
#include <io.h>
#include <time.h>

#include "xboot.h"
#include "adler32.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define VERSION         "1.00"
#define MAX_IMAGE_SIZE  (16*1024*1024)

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static char szInFile[_MAX_PATH]; /* Input file name */

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  OutputStartMessage                                                   */
/*                                                                       */
/*  Output start message.                                                */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputStartMessage (void)
{
   printf("\n");   
   printf("xbootinfo v%s compiled "__DATE__" "__TIME__"\n", VERSION);
   printf("(c) 2017 by Michael Fischer (www.emb4fun.de)\n");
   printf("\n");
} /* OutputStartMessage */

/*************************************************************************/
/*  OutputUsage                                                          */
/*                                                                       */
/*  Output "usage" message.                                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void OutputUsage (void)
{
  printf("Usage: xbootinfo -i:Infile\n");
  printf("\n");
  printf("  -i   Input file, e.g. -i:test.xbo\n");
  
} /* OutputUsage */

/*************************************************************************/
/*  DisplayXBOInfo                                                       */
/*                                                                       */
/*  Create if the "InFile" exist.                                        */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void DisplayXBOInfo (void)
{
   FILE       *hInFile;
   uint32_t    dCRC; 
   XBOOT_HEADER Header;
   struct tm    TM;

   /* Check if input file is available */
   hInFile = fopen(szInFile, "rb");
   if (NULL == hInFile)
   {
      printf("Error: Input file <%s> not found\n", szInFile);
      exit(1);
   }
   
   fread(&Header, 1, sizeof(XBOOT_HEADER), hInFile);
   fclose(hInFile);
   
   /* Check header */
   if ((XBOOT_HEADER_MAGIC_1 == Header.dMagic1) &&
       (XBOOT_HEADER_MAGIC_2 == Header.dMagic2) &&
       (XBOOT_HEADER_SIZEVER == Header.dSizeVersion))
   {
      /* Check CRC of the header */
      dCRC = adler32(ADLER_START_VALUE, (const uint8_t *)&Header, sizeof(XBOOT_HEADER) - XBOOT_SIZE_OF_CRC32);
      if (dCRC == Header.dHeaderCRC32)
      {
         TM = *localtime(&Header.dDataCreationTime);

         printf("Start of image  : 0x%08X\n", Header.dStartAddress);
         printf("Total image size: %ld bytes\n", Header.dDataTotalSize);
         printf("Creation time   : %s", asctime(&TM));
         
         if (strlen((char*)Header.DataName) < XBOOT_DATA_NAME_SIZE)
         {
            printf("Image name      : %s\n", Header.DataName);
         }
         
         if (0 == Header.dStartAddress)
         {
            printf("\nWARNING: This image will overwrite the bootloader.\n");
         } 
      }
      else
      {
         printf("Header CRC error\n");
      }
   }
   else
   {
      printf("Error: Input file <%s> is not a XBOOT file\n", szInFile);
      exit(1);
   }
   
} /* DisplayXBOInfo */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : argc, argv                                                   */
/*  Out   : none                                                         */
/*  Return: 0 / error cause                                              */
/*************************************************************************/
int main (int argc, char **argv)
{
   int   nIndex;
   int   nLen;
   char *pString;


   szInFile[0] = 0;
   
   OutputStartMessage();
   
   if (argc < 2)
   {
      OutputUsage();
   }
   else
   {
      /* 
       * Parse command line 
       */
      for (nIndex=0; nIndex<argc; nIndex++)
      {
         /* Input file */
         if ((pString = strstr(argv[nIndex], "-i:")) != NULL)
         {
            pString += 3;  /* Jump over "-i:" */
            nLen     = strlen(pString);
            if (nLen < (sizeof(szInFile)-1))
            {
               strcpy(szInFile, pString);
            }
            else
            {
               printf("Error: Input name len > %d\n", (sizeof(szInFile)-1));
               exit(1);
            }
         } /* Parse input file name */
         
      } /* end for loop */
      
      if (0 == szInFile[0])
      {
         printf("Error: No input file specified\n");
         exit(1);
      }
      
      /*
       * Here the command line was parsed and no error was available
       */
      printf("Input file : %s\n", szInFile);
      printf("\n");
      
      DisplayXBOInfo();
   }      
   
   return(0);
} /* main */

/*** EOF ***/
